import { world, system } from "@minecraft/server";

const GunReloadComponent = {
    onUse({ itemStack, source }) {
        if (!source || !itemStack) return;

        const typeId = itemStack.typeId;

        // 10mm Pistol
        if (typeId === "fw:pistol_10mm_emp") {
            system.run(() => {
                // ✅ Delay 2 seconds (40 ticks) before running reload logic
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:10mm_ammunition";
                    const ammoMax = 12;

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s pistol_10mm ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 25); // 0 ticks = 0 seconds
            });
        }

        // Pipe Rifle
        else if (typeId === "fw:pipe_rifle_emp") {
            system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before running the reload logic
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:38_ammunition";
                    const ammoMax = 12; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s pipe_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 43); // Delay = 40 ticks = 2 seconds
            });
        }

        // Sniper Rifle
        else if (typeId === "fw:sniper_rifle_emp") {
            system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before running the reload logic
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:308_ammunition";
                    const ammoMax = 10; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s sniper_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 43); // Delay = 40 ticks = 2 seconds
            });
        }

        // Classic 10mm Pistol
        else if (typeId === "fw:classic_10mm_pistol_emp") {
            system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before running the reload logic
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:10mm_ammunition";
                    const ammoMax = 8; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s classic_10mm_pistol ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 43); // Delay = 40 ticks = 2 seconds
            });
        }

        // Gauss Rifle
        else if (typeId === "fw:gauss_rifle_emp") {
            system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before running the reload logic
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:2mm_ec";
                    const ammoMax = 7; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s gauss_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 25); // Delay = 40 ticks = 2 seconds
            });
        }

        // Pipe Pistol
else if (typeId === "fw:pipe_pistol_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:38_ammunition";
                    const ammoMax = 12; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s pipe_pistol ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 30); // 0 ticks = 0 seconds
            });
        }

        // Laser Pistol
else if (typeId === "fw:laser_pistol_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:universal_fusion_cell";
                    const ammoMax = 30; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s laser_pistol ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 25); // 0 ticks = 0 seconds
            });
        }

        // Laser Rifle
else if (typeId === "fw:laser_rifle_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:universal_fusion_cell";
                    const ammoMax = 30; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s laser_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 36); // 0 ticks = 0 seconds
            });
        }

        // Institute Pistol
else if (typeId === "fw:institute_pistol_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:universal_fusion_cell";
                    const ammoMax = 30; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s institute_pistol ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 25); // 0 ticks = 0 seconds
            });
        }

        // Institute Rifle
else if (typeId === "fw:institute_rifle_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:universal_fusion_cell";
                    const ammoMax = 30; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s institute_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 36); // 0 ticks = 0 seconds
            });
        }

        // Hunting Rifle
else if (typeId === "fw:hunting_rifle_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:308_ammunition";
                    const ammoMax = 5; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s hunting_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 50); // 0 ticks = 0 seconds
            });
        }

        // Combat Rifle
else if (typeId === "fw:combat_rifle_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:5_56_ammunition";
                    const ammoMax = 20; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s combat_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 29); // 0 ticks = 0 seconds
            });
        }


        // Assault Rifle
else if (typeId === "fw:assault_rifle_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:5_56_ammunition";
                    const ammoMax = 30; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s assault_rifle ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 29); // 0 ticks = 0 seconds
            });
        }

        // Machine Gun
else if (typeId === "fw:machine_gun_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:5_56_ammunition";
                    const ammoMax = 40; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s machine_gun ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 43); // 0 ticks = 0 seconds
            });
        }

        // Minigun
else if (typeId === "fw:minigun_emp") {
    system.run(() => {
                // ✅ Wait 2 seconds (40 ticks) before starting the reload
                system.runTimeout(async () => {
                    const inventory = source.getComponent("inventory").container;
                    if (!inventory) return;

                    let totalAmmo = 0;
                    const ammoType = "fw:5mm_ammunition";
                    const ammoMax = 500; // Max ammo

                    for (let slot = 0; slot < inventory.size; slot++) {
                        const item = inventory.getItem(slot);
                        if (item && item.typeId === ammoType) {
                            totalAmmo += item.amount;
                        }
                    }

                    if (totalAmmo >= 1) {
                        const ammoToAdd = Math.min(totalAmmo, ammoMax);
                        await source.runCommandAsync(`scoreboard players add @s minigun ${ammoToAdd}`);
                        console.warn(`[RELOAD] Added ${ammoToAdd} bullets to the scoreboard.`);

                        let ammoLeftToRemove = ammoToAdd;
                        for (let slot = 0; slot < inventory.size; slot++) {
                            const item = inventory.getItem(slot);
                            if (item && item.typeId === ammoType) {
                                if (item.amount <= ammoLeftToRemove) {
                                    inventory.setItem(slot, null);
                                    ammoLeftToRemove -= item.amount;
                                } else {
                                    item.amount -= ammoLeftToRemove;
                                    inventory.setItem(slot, item);
                                    break;
                                }
                            }
                            if (ammoLeftToRemove <= 0) break;
                        }
                    } else {
                        console.warn("[RELOAD] Not enough ammo to reload.");
                    }

                }, 0); // 0 ticks = 0 seconds
            });
        }
    }
};

world.beforeEvents.worldInitialize.subscribe(({ itemComponentRegistry }) => {
    itemComponentRegistry.registerCustomComponent("fw:gun_reload", GunReloadComponent);
});
